@@ -1,58 +1,36 @@
 # -*- coding: utf-8 -*-
-# Copyright 2007-2011 The Hyperspy developers
+# Copyright 2007-2011 The HyperSpy developers
 #
-# This file is part of  Hyperspy.
+# This file is part of  HyperSpy.
 #
-#  Hyperspy is free software: you can redistribute it and/or modify
+#  HyperSpy is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
-#  Hyperspy is distributed in the hope that it will be useful,
+#  HyperSpy is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with  Hyperspy.  If not, see <http://www.gnu.org/licenses/>.
+# along with  HyperSpy.  If not, see <http://www.gnu.org/licenses/>.
 
 import copy
 
 import numpy as np
 import traits.api as t
-import traitsui.api as tui
 from traits.trait_errors import TraitError
 
+from hyperspy.misc.utils import isiterable, ordinal
+
+
 class ndindex_nat(np.ndindex):
+
     def next(self):
         return super(ndindex_nat, self).next()[::-1]
 
 
-def get_axis_group(n , label=''):
-    group = tui.Group(
-            tui.Group(
-                tui.Item('axis%i.name' % n),
-                tui.Item('axis%i.size' % n, style='readonly'),
-                tui.Item('axis%i.index_in_array' % n, style='readonly'),
-                tui.Item('axis%i.low_index' % n, style='readonly'),
-                tui.Item('axis%i.high_index' % n, style='readonly'),
-                # The style of the index is chosen to be readonly because of 
-                # a bug in Traits 4.0.0 when using context with a Range traits
-                # where the limits are defined by another traits_view
-                tui.Item('axis%i.index' % n, style='readonly'),
-                tui.Item('axis%i.value' % n, style='readonly'),
-                tui.Item('axis%i.units' % n),
-                tui.Item('axis%i.navigate' % n, label = 'slice'),
-            show_border = True,),
-            tui.Group(
-                tui.Item('axis%i.scale' % n),
-                tui.Item('axis%i.offset' % n),
-            label = 'Calibration',
-            show_border = True,),
-        label = label,
-        show_border = True,)
-    return group
-    
 def generate_axis(offset, scale, size, offset_index=0):
     """Creates an axis given the offset, scale and number of channels
 
@@ -69,14 +47,15 @@ def generate_axis(offset, scale, size, offset_index=0):
     Returns
     -------
     Numpy array
-    
+
     """
     return np.linspace(offset - offset_index * scale,
                        offset + scale * (size - 1 - offset_index),
                        size)
 
+
 class DataAxis(t.HasTraits):
-    name=t.Str()
+    name = t.Str()
     units = t.Str()
     scale = t.Float()
     offset = t.Float()
@@ -95,12 +74,11 @@ class DataAxis(t.HasTraits):
     def __init__(self,
                  size,
                  index_in_array=None,
-                 name='',
+                 name=t.Undefined,
                  scale=1.,
                  offset=0.,
-                 units='undefined',
+                 units=t.Undefined,
                  navigate=t.Undefined):
-                     
         super(DataAxis, self).__init__()
         self.name = name
         self.units = units
@@ -119,10 +97,10 @@ class DataAxis(t.HasTraits):
         self.on_trait_change(self.set_index_from_value, 'value')
         self.on_trait_change(self._update_slice, 'navigate')
         self.on_trait_change(self.update_index_bounds, 'size')
-        # The slice must be updated even if the default value did not 
+        # The slice must be updated even if the default value did not
         # change to correctly set its value.
         self._update_slice(self.navigate)
-    
+
     @property
     def index_in_array(self):
         if self.axes_manager is not None:
@@ -132,42 +110,43 @@ class DataAxis(t.HasTraits):
                 "This DataAxis does not belong to an AxesManager"
                 " and therefore its index_in_array attribute "
                 " is not defined")
+
     @property
     def index_in_axes_manager(self):
         if self.axes_manager is not None:
             return self.axes_manager._get_axes_in_natural_order().\
-                   index(self)
+                index(self)
         else:
             raise AttributeError(
                 "This DataAxis does not belong to an AxesManager"
                 " and therefore its index_in_array attribute "
                 " is not defined")
-                        
+
     def _get_positive_index(self, index):
         if index < 0:
             index = self.size + index
             if index < 0:
                 raise IndexError("index out of bounds")
         return index
-        
+
     def _get_index(self, value):
         if isinstance(value, float):
             return self.value2index(value)
         else:
             return value
-        
+
     def _slice_me(self, slice_):
-        """Returns a slice to slice the corresponding data axis and 
+        """Returns a slice to slice the corresponding data axis and
         change the offset and scale of the DataAxis acordingly.
-        
+
         Parameters
         ----------
         slice_ : {float, int, slice}
-        
+
         Returns
         -------
         my_slice : slice
-        
+
         """
         i2v = self.index2value
         v2i = self.value2index
@@ -183,19 +162,29 @@ class DataAxis(t.HasTraits):
                 start = self._get_positive_index(slice_)
             stop = start + 1
             step = None
-            
+
         if isinstance(step, float):
             step = int(round(step / self.scale))
         if isinstance(start, float):
-            start = v2i(start)
+            try:
+                start = v2i(start)
+            except ValueError:
+                # The value is below the axis limits
+                # we slice from the start.
+                start = None
         if isinstance(stop, float):
-            stop = v2i(stop) 
-            
+            try:
+                stop = v2i(stop)
+            except ValueError:
+                # The value is above the axes limits
+                # we slice up to the end.
+                stop = None
+
         if step == 0:
             raise ValueError("slice step cannot be zero")
-            
+
         my_slice = slice(start, stop, step)
-        
+
         if start is None:
             if step > 0 or step is None:
                 start = 0
@@ -204,19 +193,26 @@ class DataAxis(t.HasTraits):
         self.offset = i2v(start)
         if step is not None:
             self.scale *= step
-            
+
         return my_slice
 
+    def _get_name(self):
+        name = (self.name if self.name is not t.Undefined
+                else ("Unnamed " +
+                      ordinal(self.index_in_axes_manager)))
+        return name
 
     def __repr__(self):
-        if self.name is not None:
-            text = '<%s axis, size: %i' % (self.name,
-                                              self.size,)
-            if self.navigate is True:
-                text += ", index: %i" % self.index
-            text += ">"
-            return text
-            
+        text = '<%s axis, size: %i' % (self._get_name(),
+                                       self.size,)
+        if self.navigate is True:
+            text += ", index: %i" % self.index
+        text += ">"
+        return text
+
+    def __str__(self):
+        return self._get_name() + " axis"
+
     def connect(self, f, trait='value'):
         self.on_trait_change(f, trait)
 
@@ -240,25 +236,30 @@ class DataAxis(t.HasTraits):
 
     def get_axis_dictionary(self):
         adict = {
-            'name' : self.name,
-            'scale' : self.scale,
-            'offset' : self.offset,
-            'size' : self.size,
-            'units' : self.units,
-            'index_in_array' : self.index_in_array,
-            'navigate' : self.navigate
+            'name': self.name,
+            'scale': self.scale,
+            'offset': self.offset,
+            'size': self.size,
+            'units': self.units,
+            'navigate': self.navigate
         }
         return adict
-        
+
     def copy(self):
         return DataAxis(**self.get_axis_dictionary())
 
+    def __copy__(self):
+        return self.copy()
+
+    def __deepcopy__(self, memo):
+        cp = self.copy()
+        return cp
+
     def update_value(self):
         self.value = self.axis[self.index]
 
-    def value2index(self, value):
-        """Return the closest index to the given value if between the limits,
-        otherwise it will return either the upper or lower limits
+    def value2index(self, value, rounding=round):
+        """Return the closest index to the given value if between the limit.
 
         Parameters
         ----------
@@ -267,21 +268,27 @@ class DataAxis(t.HasTraits):
         Returns
         -------
         int
+
+        Raises
+        ------
+        ValueError if value is out of the axis limits.
+
         """
         if value is None:
             return None
         else:
-            index = int(round((value - self.offset) / \
-            self.scale))
+            index = int(rounding((value - self.offset) /
+                                 self.scale))
             if self.size > index >= 0:
                 return index
-            elif index < 0:
-                return 0
             else:
-                return int(self.size - 1)
+                raise ValueError("The value is out of the axis limits")
 
     def index2value(self, index):
-        return self.axis[index]
+        if isinstance(index, np.ndarray):
+            return self.axis[index.ravel()].reshape(index.shape)
+        else:
+            return self.axis[index]
 
     def set_index_from_value(self, value):
         self.index = self.value2index(value)
@@ -289,9 +296,9 @@ class DataAxis(t.HasTraits):
         if self.continuous_value is False:
             self.value = self.index2value(self.index)
 
-    def calibrate(self, value_tuple, index_tuple, modify_calibration = True):
+    def calibrate(self, value_tuple, index_tuple, modify_calibration=True):
         scale = (value_tuple[1] - value_tuple[0]) /\
-        (index_tuple[1] - index_tuple[0])
+            (index_tuple[1] - index_tuple[0])
         offset = value_tuple[0] - scale * index_tuple[0]
         if modify_calibration is True:
             self.offset = offset
@@ -299,127 +306,156 @@ class DataAxis(t.HasTraits):
         else:
             return offset, scale
 
-    traits_view = \
-    tui.View(
-        tui.Group(
-            tui.Group(
-                tui.Item(name='name'),
-                tui.Item(name='size', style='readonly'),
-                tui.Item(name='index_in_array', style='readonly'),
-                tui.Item(name='index'),
-                tui.Item(name='value', style='readonly'),
-                tui.Item(name='units'),
-                tui.Item(name='navigate', label = 'navigate'),
-            show_border = True,),
-            tui.Group(
-                tui.Item(name='scale'),
-                tui.Item(name='offset'),
-            label = 'Calibration',
-            show_border = True,),
-        label = "Data Axis properties",
-        show_border = True,),
-    title = 'Axis configuration',
-    )
+    def value_range_to_indices(self, v1, v2):
+        """Convert the given range to index range.
+
+        When an out of the axis limits, the endpoint is used instead.
+
+        Parameters
+        ----------
+        v1, v2 : float
+            The end points of the interval in the axis units. v2 must be
+            greater than v1.
+
+        """
+        if v1 > v2:
+            raise ValueError("v2 must be greater than v1.")
+
+        if v1 is not None and v1 > self.low_value and v1 <= self.high_value:
+            i1 = self.value2index(v1)
+        else:
+            i1 = 0
+        if v2 is not None and v2 < self.high_value and v2 >= self.low_value:
+            i2 = self.value2index(v2)
+        else:
+            i2 = self.size - 1
+        return i1, i2
+
 
 class AxesManager(t.HasTraits):
+
     """Contains and manages the data axes.
-    
-    It can iterate over the navigation coordinates returning the 
-    indices at the current iteration.
-    
-    It can only be indexed and sliced to access the DataAxis objects 
-    that it contain. The indexing is in the same "natural order" as in 
-    Signal, i.e. [nX, nY, ...,sX, sY,...] where `n` indicates a navigation axis and 
-    `s` a signal axis. In addition it can be indexed using the DataAxis
-    name.
-    
-    
+
+    It supports indexing, slicing, subscriptins and iteration. As an interator,
+    iterate over the navigation coordinates returning the current indices.
+    It can only be indexed and sliced to access the DataAxis objects that it
+    contains. Standard indexing and slicing follows the "natural order" as in
+    Signal, i.e. [nX, nY, ...,sX, sY,...] where `n` indicates a navigation axis
+    and `s` a signal axis. In addition AxesManager support indexing using
+    complex numbers a + bj, where b can be one of 0, 1, 2 and 3 and a a valid
+    index. If b is 3 AxesManager is indexed using the order of the axes in the
+    array. If b is 1(2), indexes only the navigation(signal) axes in the
+    natural order. In addition AxesManager supports subscription using
+    axis name.
+
     Attributes
     ----------
-    
+
     coordinates : tuple
         Get and set the current coordinates if the navigation dimension
-        is not 0. If the navigation dimension is 0 it raises 
+        is not 0. If the navigation dimension is 0 it raises
         AttributeError when attempting to set its value.
 
-    
+
     indices : tuple
         Get and set the current indices if the navigation dimension
-        is not 0. If the navigation dimension is 0 it raises 
+        is not 0. If the navigation dimension is 0 it raises
         AttributeError when attempting to set its value.
-        
+
     signal_axes, navigation_axes : list
-    	Contain the corresponding DataAxis objects
+        Contain the corresponding DataAxis objects
 
     Examples
     --------
-    
+
     >>> import numpy as np
-    
+
     Create a spectrum with random data
-    
-    >>> s = signals.Spectrum(np.random.random((2,2,2,10)))
+
+    >>> s = signals.Spectrum(np.random.random((2,3,4,5)))
     >>> s.axes_manager
-    <Axes manager, 4 axes, signal dimension: 1, navigation dimension: 3>
-    
-    >>> s.axes_manager[1]
-    <undefined navigation axis, size: 2, index: 0>
+    <Axes manager, axes: (<axis2 axis, size: 4, index: 0>, <axis1 axis, size: 3, index: 0>, <axis0 axis, size: 2, index: 0>, <axis3 axis, size: 5>)>
+    >>> s.axes_manager[0]
+    <axis2 axis, size: 4, index: 0>
+    >>> s.axes_manager[3j]
+    <axis0 axis, size: 2, index: 0>
+    >>> s.axes_manager[1j]
+    <axis2 axis, size: 4, index: 0>
+    >>> s.axes_manager[2j]
+    <axis3 axis, size: 5>
     >>> s.axes_manager[1].name="y"
     >>> s.axes_manager['y']
-    <y navigation axis, size: 2, index: 0>
+    <y axis, size: 3 index: 0>
     >>> for i in s.axes_manager:
     >>>     print i, s.axes_manager.indices
     (0, 0, 0) (0, 0, 0)
-    (0, 0, 1) (0, 0, 1)
-    (0, 1, 0) (0, 1, 0)
-    (0, 1, 1) (0, 1, 1)
     (1, 0, 0) (1, 0, 0)
-    (1, 0, 1) (1, 0, 1)
+    (2, 0, 0) (2, 0, 0)
+    (3, 0, 0) (3, 0, 0)
+    (0, 1, 0) (0, 1, 0)
     (1, 1, 0) (1, 1, 0)
+    (2, 1, 0) (2, 1, 0)
+    (3, 1, 0) (3, 1, 0)
+    (0, 2, 0) (0, 2, 0)
+    (1, 2, 0) (1, 2, 0)
+    (2, 2, 0) (2, 2, 0)
+    (3, 2, 0) (3, 2, 0)
+    (0, 0, 1) (0, 0, 1)
+    (1, 0, 1) (1, 0, 1)
+    (2, 0, 1) (2, 0, 1)
+    (3, 0, 1) (3, 0, 1)
+    (0, 1, 1) (0, 1, 1)
     (1, 1, 1) (1, 1, 1)
-    
+    (2, 1, 1) (2, 1, 1)
+    (3, 1, 1) (3, 1, 1)
+    (0, 2, 1) (0, 2, 1)
+    (1, 2, 1) (1, 2, 1)
+    (2, 2, 1) (2, 2, 1)
+    (3, 2, 1) (3, 2, 1)
+
     """
+
     _axes = t.List(DataAxis)
     signal_axes = t.Tuple()
     navigation_axes = t.Tuple()
     _step = t.Int(1)
-    
+
     def __init__(self, axes_list):
         super(AxesManager, self).__init__()
         self.create_axes(axes_list)
-        # set_signal_dimension is called only if there is no current 
+        # set_signal_dimension is called only if there is no current
         # view. It defaults to spectrum
         navigates = [i.navigate for i in self._axes]
         if t.Undefined in navigates:
             # Default to Spectrum view if the view is not fully defined
             self.set_signal_dimension(1)
-        
+
         self._update_attributes()
         self.on_trait_change(self._update_attributes, '_axes.slice')
         self.on_trait_change(self._update_attributes, '_axes.index')
         self.on_trait_change(self._update_attributes, '_axes.size')
-        self._index = None # index for the iterator
-    
+        self._index = None  # index for the iterator
+
     def _get_positive_index(self, axis):
         if axis < 0:
             axis = len(self._axes) + axis
             if axis < 0:
                 raise IndexError("index out of bounds")
         return axis
-        
+
     def _array_indices_generator(self):
         shape = (self.navigation_shape[::-1] if self.navigation_size > 0 else
-                 [1,])
+                 [1, ])
         return np.ndindex(*shape)
-        
+
     def _am_indices_generator(self):
         shape = (self.navigation_shape if self.navigation_size > 0 else
-                 [1,])[::-1]
+                 [1, ])[::-1]
         return ndindex_nat(*shape)
-    
+
     def __getitem__(self, y):
         """x.__getitem__(y) <==> x[y]
-        
+
         """
         if isinstance(y, basestring):
             axes = list(self._get_axes_in_natural_order())
@@ -428,88 +464,100 @@ class AxesManager(t.HasTraits):
                 if y == axis.name:
                     return axis
             raise ValueError("There is no DataAxis named %s" % y)
-        else:
-            # Use the "natural order" as in Signal
+        elif (isinstance(y.real, float) and not y.real.is_integer() or
+                isinstance(y.imag, float) and not y.imag.is_integer()):
+            raise TypeError("axesmanager indices must be integers, "
+                            "complex intergers or strings")
+        if y.imag == 0:  # Natural order
             return self._get_axes_in_natural_order()[y]
-        
+        elif y.imag == 3:  # Array order
+            # Array order
+            return self._axes[int(y.real)]
+        elif y.imag == 1:  # Navigation natural order
+            #
+            return self.navigation_axes[int(y.real)]
+        elif y.imag == 2:  # Signal natural order
+            return self.signal_axes[int(y.real)]
+        else:
+            raise IndexError("axesmanager imaginary part of complex indices "
+                             "must be 0, 1 or 2")
+
     def __getslice__(self, i=None, j=None):
         """x.__getslice__(i, j) <==> x[i:j]
-        
+
         """
         return self._get_axes_in_natural_order()[i:j]
-        
+
     def _get_axes_in_natural_order(self):
         return self.navigation_axes + self.signal_axes
-        
-    @property            
+
+    @property
     def _navigation_shape_in_array(self):
         return self.navigation_shape[::-1]
-        
-    @property            
+
+    @property
     def _signal_shape_in_array(self):
         return self.signal_shape[::-1]
-    @property    
+
+    @property
     def shape(self):
         nav_shape = (self.navigation_shape
                      if self.navigation_shape != (0,)
                      else tuple())
         sig_shape = (self.signal_shape
-             if self.signal_shape != (0,)
-             else tuple())
+                     if self.signal_shape != (0,)
+                     else tuple())
         return nav_shape + sig_shape
-        
+
     def remove(self, axis):
         """Remove the given Axis.
-        
+
         Raises
         ------
         ValueError if the Axis is not present.
-        
+
         """
-        if axis not in self._axes:
-            raise ValueError(
-                "AxesManager.remove(x): x not in AxesManager")
+        axis = self[axis]
         axis.axes_manager = None
         self._axes.remove(axis)
-            
+
     def __delitem__(self, i):
         self.remove(self[i])
-        
+
     def _get_data_slice(self, fill=None):
         """Return a tuple of slice objects to slice the data.
-        
+
         Parameters
         ----------
         fill: None or iterable of (int, slice)
             If not None, fill the tuple of index int with the given
             slice.
-            
+
         """
-        cslice = [slice(None),] * len(self._axes)
+        cslice = [slice(None), ] * len(self._axes)
         if fill is not None:
             for index, slice_ in fill:
                 cslice[index] = slice_
         return tuple(cslice)
-        
-        
+
     def create_axes(self, axes_list):
         """Given a list of dictionaries defining the axes properties
         create the DataAxis instances and add them to the AxesManager.
-        
-        The index of the axis in the array and in the `_axes` lists 
-        can be defined by the index_in_array keyword if given 
-        for all axes. Otherwise it is defined by their index in the 
+
+        The index of the axis in the array and in the `_axes` lists
+        can be defined by the index_in_array keyword if given
+        for all axes. Otherwise it is defined by their index in the
         list.
-        
+
         See also
         --------
         append_axis
-        
+
         """
         # Reorder axes_list using index_in_array if it is defined
         # for all axes and the indices are not repeated.
         indices = set([axis['index_in_array'] for axis in axes_list if
-                   'index_in_array' in axis])
+                       'index_in_array' in axis])
         if len(indices) == len(axes_list):
             axes_list.sort(key=lambda x: x['index_in_array'])
         for axis_dict in axes_list:
@@ -524,39 +572,35 @@ class AxesManager(t.HasTraits):
 
     def next(self):
         """
-        Standard iterator method, updates the index and returns the 
+        Standard iterator method, updates the index and returns the
         current coordiantes
 
         Returns
         -------
         val : tuple of ints
-            Returns a tuple containing the coordiantes of the current 
+            Returns a tuple containing the coordiantes of the current
             iteration.
 
         """
         if self._index is None:
             self._index = 0
-            self._indices_backup = self.indices
             val = (0,) * self.navigation_dimension
             self.indices = val
         elif (self._index >= self._max_index):
-            self._index = None
-            self.indices = self._indices_backup
-            del self._indices_backup
             raise StopIteration
         else:
             self._index += 1
-            val = np.unravel_index(self._index, 
+            val = np.unravel_index(self._index,
                                    tuple(self._navigation_shape_in_array))[::-1]
             self.indices = val
         return val
 
     def __iter__(self):
-        # Reset the _index that can have a value != None due to 
+        # Reset the _index that can have a value != None due to
         # a previous iteration that did not hit a StopIteration
         self._index = None
         return self
-        
+
     def append_axis(self, *args, **kwargs):
         axis = DataAxis(*args, **kwargs)
         axis.axes_manager = self
@@ -589,7 +633,7 @@ class AxesManager(t.HasTraits):
                 axis.size for axis in self.navigation_axes])
         else:
             self.navigation_shape = (0,)
-            
+
         if self.signal_dimension != 0:
             self.signal_shape = tuple([
                 axis.size for axis in self.signal_axes])
@@ -600,33 +644,33 @@ class AxesManager(t.HasTraits):
         self.signal_size = \
             np.cumprod(self.signal_shape)[-1]
         self._update_max_index()
-            
+
     def set_signal_dimension(self, value):
         """Set the dimension of the signal.
-                
+
         Attributes
         ----------
         value : int
-        
+
         Raises
         ------
-        ValueError if value if greater than the number of axes or 
-        is negative         
-        
+        ValueError if value if greater than the number of axes or
+        is negative
+
         """
         if len(self._axes) == 0:
             return
         elif value > len(self._axes):
             raise ValueError("The signal dimension cannot be greater"
-                " than the number of axes which is %i" % len(self._axes))
+                             " than the number of axes which is %i" % len(self._axes))
         elif value < 0:
             raise ValueError(
                 "The signal dimension must be a positive integer")
-                
+
         tl = [True] * len(self._axes)
         if value != 0:
             tl[-value:] = (False,) * value
-        
+
         for axis in self._axes:
             axis.navigate = tl.pop(0)
 
@@ -641,7 +685,8 @@ class AxesManager(t.HasTraits):
                 axis.on_trait_change(f, 'index', remove=True)
 
     def key_navigator(self, event):
-        if len(self.navigation_axes) not in (1,2): return
+        if len(self.navigation_axes) not in (1, 2):
+            return
         x = self.navigation_axes[0]
         try:
             if event.key == "right" or event.key == "6":
@@ -663,8 +708,9 @@ class AxesManager(t.HasTraits):
             pass
 
     def gui(self):
+        from hyperspy.gui.axes import data_axis_view
         for axis in self._axes:
-            axis.edit_traits()
+            axis.edit_traits(view=data_axis_view)
 
     def copy(self):
         return(copy.copy(self))
@@ -680,102 +726,144 @@ class AxesManager(t.HasTraits):
         for axis in self._axes:
             axes_dicts.append(axis.get_axis_dictionary())
         return axes_dicts
-        
+
     def as_dictionary(self):
         am_dict = {}
         for i, axis in enumerate(self._axes):
             am_dict['axis-%i' % i] = axis.get_axis_dictionary()
         return am_dict
-        
+
     def _get_signal_axes_dicts(self):
-        return [axis.get_axis_dictionary() for axis in 
+        return [axis.get_axis_dictionary() for axis in
                 self.signal_axes[::-1]]
 
     def _get_navigation_axes_dicts(self):
-        return [axis.get_axis_dictionary() for axis in 
+        return [axis.get_axis_dictionary() for axis in
                 self.navigation_axes[::-1]]
-        
+
     def show(self):
+        from hyperspy.gui.axes import get_axis_group
+        import traitsui.api as tui
         context = {}
         ag = []
-        for n in range(0,len(self._axes)):
-            ag.append(get_axis_group(n, self._axes[n].name))
-            context['axis%i' % n] = self._axes[n]
+        for n, axis in enumerate(self._get_axes_in_natural_order()):
+            ag.append(get_axis_group(n, str(axis)))
+            context['axis%i' % n] = axis
         ag = tuple(ag)
-        self.edit_traits(view = tui.View(*ag), context = context)
-        
+        self.edit_traits(view=tui.View(*ag), context=context)
+
+    def _get_axes_str(self):
+        string = "("
+        for axis in self.navigation_axes:
+            string += axis.__repr__() + ", "
+        string = string.rstrip(", ")
+        string += "|"
+        for axis in self.signal_axes:
+            string += axis.__repr__() + ", "
+        string = string.rstrip(", ")
+        string += ")"
+        return string
+
+    def _get_dimension_str(self):
+        string = "("
+        for axis in self.navigation_axes:
+            string += str(axis.size) + ", "
+        string = string.rstrip(", ")
+        string += "|"
+        for axis in self.signal_axes:
+            string += str(axis.size) + ", "
+        string = string.rstrip(", ")
+        string += ")"
+        return string
+
     def __repr__(self):
-        text = ('<Axes manager, axes: %s>' % 
-            self._get_axes_in_natural_order().__repr__())
-            
+        text = ('<Axes manager, axes: %s>' %
+                self._get_axes_str())
         return text
-    
-    @property        
+
+    @property
     def coordinates(self):
         """Get the coordinates of the navigation axes.
-        
+
         Returns
         -------
         list
-            
+
         """
         return tuple([axis.value for axis in self.navigation_axes])
-        
-    @coordinates.setter    
+
+    @coordinates.setter
     def coordinates(self, coordinates):
         """Set the coordinates of the navigation axes.
-        
+
         Parameters
         ----------
         coordinates : tuple
             The len of the the tuple must coincide with the navigation
             dimension
-            
+
         """
-        
+
         if len(coordinates) != self.navigation_dimension:
             raise AttributeError(
-            "The number of coordinates must be equal to the "
-            "navigation dimension that is %i" % 
+                "The number of coordinates must be equal to the "
+                "navigation dimension that is %i" %
                 self.navigation_dimension)
         for value, axis in zip(coordinates, self.navigation_axes):
             axis.value = value
-            
-    @property        
+
+    @property
     def indices(self):
         """Get the index of the navigation axes.
-        
+
         Returns
         -------
         list
-            
+
         """
         return tuple([axis.index for axis in self.navigation_axes])
-        
-    @indices.setter    
+
+    @indices.setter
     def indices(self, indices):
         """Set the index of the navigation axes.
-        
+
         Parameters
         ----------
         indices : tuple
             The len of the the tuple must coincide with the navigation
             dimension
-            
+
         """
-        
+
         if len(indices) != self.navigation_dimension:
             raise AttributeError(
-            "The number of indices must be equal to the "
-            "navigation dimension that is %i" % 
+                "The number of indices must be equal to the "
+                "navigation dimension that is %i" %
                 self.navigation_dimension)
         for index, axis in zip(indices, self.navigation_axes):
             axis.index = index
-            
+
     def _get_axis_attribute_values(self, attr):
         return [getattr(axis, attr) for axis in self._axes]
-    
+
     def _set_axis_attribute_values(self, attr, values):
-        for axis, value in zip(self._axes,values):
+        """Set the given attribute of all the axes to the given
+        value(s)
+
+        Parameters
+        ----------
+        attr : string
+            The DataAxis attribute to set.
+        values: any
+            If iterable, it must have the same number of items
+            as axes are in this AxesManager instance. If not iterable,
+            the attribute of all the axes are set to the given value.
+
+        """
+        if not isiterable(values):
+            values = [values, ] * len(self._axes)
+        elif len(values) != len(self._axes):
+            raise ValueError("Values must have the same number"
+                             "of items are axes are in this AxesManager")
+        for axis, value in zip(self._axes, values):
             setattr(axis, attr, value)
-            
